@@ -19,8 +19,11 @@
 
 #include "config.h"
 
+#include "backends/edid.h"
 #include "backends/meta-output.h"
 
+#include "backends/meta-crtc.h"
+
 enum
 {
   PROP_0,
@@ -42,6 +45,8 @@ typedef struct _MetaOutputPrivate
 
   MetaOutputInfo *info;
 
+  MetaMonitor *monitor;
+
   /* The CRTC driving this output, NULL if the output is not enabled */
   MetaCrtc *crtc;
 
@@ -109,6 +114,37 @@ meta_output_get_gpu (MetaOutput *output)
   return priv->gpu;
 }
 
+MetaMonitor *
+meta_output_get_monitor (MetaOutput *output)
+{
+  MetaOutputPrivate *priv = meta_output_get_instance_private (output);
+
+  g_warn_if_fail (priv->monitor);
+
+  return priv->monitor;
+}
+
+void
+meta_output_set_monitor (MetaOutput  *output,
+                         MetaMonitor *monitor)
+{
+  MetaOutputPrivate *priv = meta_output_get_instance_private (output);
+
+  g_warn_if_fail (!priv->monitor);
+
+  priv->monitor = monitor;
+}
+
+void
+meta_output_unset_monitor (MetaOutput *output)
+{
+  MetaOutputPrivate *priv = meta_output_get_instance_private (output);
+
+  g_warn_if_fail (priv->monitor);
+
+  priv->monitor = NULL;
+}
+
 const char *
 meta_output_get_name (MetaOutput *output)
 {
@@ -190,8 +226,12 @@ meta_output_assign_crtc (MetaOutput                 *output,
 
   g_assert (crtc);
 
+  meta_output_unassign_crtc (output);
+
   g_set_object (&priv->crtc, crtc);
 
+  meta_crtc_assign_output (crtc, output);
+
   priv->is_primary = output_assignment->is_primary;
   priv->is_presentation = output_assignment->is_presentation;
   priv->is_underscanning = output_assignment->is_underscanning;
@@ -202,7 +242,11 @@ meta_output_unassign_crtc (MetaOutput *output)
 {
   MetaOutputPrivate *priv = meta_output_get_instance_private (output);
 
-  g_clear_object (&priv->crtc);
+  if (priv->crtc)
+    {
+      meta_crtc_unassign_output (priv->crtc, output);
+      g_clear_object (&priv->crtc);
+    }
 
   priv->is_primary = FALSE;
   priv->is_presentation = FALSE;
@@ -241,6 +285,68 @@ meta_output_crtc_to_logical_transform (MetaOutput           *output,
                                            inverted_panel_orientation_transform);
 }
 
+void
+meta_output_info_parse_edid (MetaOutputInfo *output_info,
+                             GBytes         *edid)
+{
+  MonitorInfo *parsed_edid;
+  size_t len;
+
+  if (!edid)
+    goto out;
+
+  parsed_edid = decode_edid (g_bytes_get_data (edid, &len));
+
+  if (parsed_edid)
+    {
+      output_info->vendor = g_strndup (parsed_edid->manufacturer_code, 4);
+      if (!g_utf8_validate (output_info->vendor, -1, NULL))
+        g_clear_pointer (&output_info->vendor, g_free);
+
+      output_info->product = g_strndup (parsed_edid->dsc_product_name, 14);
+      if (!g_utf8_validate (output_info->product, -1, NULL) ||
+          output_info->product[0] == '\0')
+        {
+          g_clear_pointer (&output_info->product, g_free);
+          output_info->product = g_strdup_printf ("0x%04x", (unsigned) parsed_edid->product_code);
+        }
+
+      output_info->serial = g_strndup (parsed_edid->dsc_serial_number, 14);
+      if (!g_utf8_validate (output_info->serial, -1, NULL) ||
+          output_info->serial[0] == '\0')
+        {
+          g_clear_pointer (&output_info->serial, g_free);
+          output_info->serial = g_strdup_printf ("0x%08x", parsed_edid->serial_number);
+        }
+
+      g_free (parsed_edid);
+    }
+
+ out:
+  if (!output_info->vendor)
+    output_info->vendor = g_strdup ("unknown");
+  if (!output_info->product)
+    output_info->product = g_strdup ("unknown");
+  if (!output_info->serial)
+    output_info->serial = g_strdup ("unknown");
+}
+
+gboolean
+meta_output_is_laptop (MetaOutput *output)
+{
+  const MetaOutputInfo *output_info = meta_output_get_info (output);
+
+  switch (output_info->connector_type)
+    {
+    case META_CONNECTOR_TYPE_eDP:
+    case META_CONNECTOR_TYPE_LVDS:
+    case META_CONNECTOR_TYPE_DSI:
+      return TRUE;
+    default:
+      return FALSE;
+    }
+}
+
 static void
 meta_output_set_property (GObject      *object,
                           guint         prop_id,
@@ -357,3 +463,37 @@ meta_output_class_init (MetaOutputClass *klass)
                         G_PARAM_STATIC_STRINGS);
   g_object_class_install_properties (object_class, N_PROPS, obj_props);
 }
+
+gboolean
+meta_tile_info_equal (MetaTileInfo *a,
+                      MetaTileInfo *b)
+{
+  if (a == b)
+    return TRUE;
+
+  if (a->group_id != b->group_id)
+    return FALSE;
+
+  if (a->flags != b->flags)
+    return FALSE;
+
+  if (a->max_h_tiles != b->max_h_tiles)
+    return FALSE;
+
+  if (a->max_v_tiles != b->max_v_tiles)
+    return FALSE;
+
+  if (a->loc_h_tile != b->loc_h_tile)
+    return FALSE;
+
+  if (a->loc_v_tile != b->loc_v_tile)
+    return FALSE;
+
+  if (a->tile_w != b->tile_w)
+    return FALSE;
+
+  if (a->tile_h != b->tile_h)
+    return FALSE;
+
+  return TRUE;
+}
